// AVAILABLE CHECK TYPES
//    PICOTEST_ASSERT(check1);
//    PICOTEST_VERIFY(0, "BETA");
//    PICOTEST_ABORT();

bool lineTests() {
    return smTestGridTxt(12,
        "                    ",
        "     *              ",
        "     *              ",
        "     *         *    ",
        "     *         *    ",
        "   ********    *    ",
        "     *         *    ",
        "     *         *    ",
        "     *         *    ",
        " ****************** ",
        "     *         *    ",
        "                    ");
}

bool limitTests() {
    return smTestGridTxt(12,
        "********************",
        "     *              ",
        "     *              ",
        "     *              ",
        "     *              ",
        "     *              ",
        "     *              ",
        "     *              ",
        "     *              ",
        "     *              ",
        "     *         *    ",
        "     *              ");
}

#define H 12
#define W 20

PICOTEST_SUITE(mainSuite, LINE_TESTS, LIMIT_TESTS, REVERSE_TESTS)
PICOTEST_CASE(LINE_TESTS) {
    smGrid(W,H);
    drawHLine(5,3,10);
    drawVLine(15,3,10);
    drawALine(5,1,10,true);
    drawALine(1,9,18,false);
    smGrid2NamedImage("LINES.bmp");
    PICOTEST_VERIFY(lineTests(), "Line test failed!")
}
PICOTEST_CASE(LIMIT_TESTS) {
    smGrid(W,H);
    PICOTEST_VERIFY( !drawHLine(-1,2,5), "Horizontal with y < 0" )
    PICOTEST_VERIFY( !drawHLine(H,2,5), "Horizontal with y >= H" )
    PICOTEST_VERIFY( !drawHLine(5,-1,5), "Horizontal with x1 < 0" )
    PICOTEST_VERIFY( !drawHLine(5,5,W), "Horizontal with x2 >= W" )

    PICOTEST_VERIFY( !drawVLine(-1,2,5), "Vertical with x < 0" )
    PICOTEST_VERIFY( !drawVLine(W,2,5), "Vertical with x >= W" )
    PICOTEST_VERIFY( !drawVLine(5,-1,5), "Vertical with y1 < 0" )
    PICOTEST_VERIFY( !drawVLine(5,5,H), "Vertical with y2 >= H" )

    PICOTEST_VERIFY( !drawALine(-1,5,4,true), "Vertical ALine with x < 0" )
    PICOTEST_VERIFY( !drawALine(-1,5,4,false), "Horizontal ALine with x < 0" )
    PICOTEST_VERIFY( !drawALine(W,5,4,true), "Vertical ALine with x >= W" )
    PICOTEST_VERIFY( !drawALine(W,5,4,false), "Horizontal ALine with x >= W" )

    PICOTEST_VERIFY( !drawALine(5,-1,4,true), "Vertical ALine with y < 0" )
    PICOTEST_VERIFY( !drawALine(5,-1,4,false), "Horizontal ALine with y < 0" )
    PICOTEST_VERIFY( !drawALine(5,H,4,true), "Vertical ALine with y >= H" )
    PICOTEST_VERIFY( !drawALine(5,H,4,false), "Horizontal ALine with y >= H" )

    PICOTEST_VERIFY( drawALine(5,0,H,true), "Vertical ALine L just acceptable" )
    PICOTEST_VERIFY( !drawALine(0,5,H+1,true), "Vertical ALine L just too long" )
    PICOTEST_VERIFY( drawALine(0,0,W,false), "Horizontal ALine L just acceptable" )
    PICOTEST_VERIFY( !drawALine(10,0,W+1,false), "Horizontal ALine L just too long" )

    PICOTEST_VERIFY( drawALine(15,10,1,true), "Vertical ALine L = 1" )
    PICOTEST_VERIFY( drawALine(15,10,1,false), "Horizontal ALine L = 1" )

    PICOTEST_VERIFY( drawALine(10,10,0,true), "Vertical ALine L = 0" )
    PICOTEST_VERIFY( drawALine(10,10,0,false), "Horizontal ALine L = 0" )

    PICOTEST_VERIFY( !drawALine(10,5,-1,true), "Vertical ALine L < 0 but in bounds" )
    PICOTEST_VERIFY( !drawALine(10,5,-1,false), "Horizontal ALine L < 0 but in bounds" )

    PICOTEST_VERIFY( !drawALine(5,5,-8,true), "Vertical ALine L < 0 but out of bounds" )
    PICOTEST_VERIFY( !drawALine(5,5,-8,false), "Horizontal ALine L < 0 but out of bounds" )

    PICOTEST_VERIFY(limitTests(), "Limit test failed!")
    smGrid2NamedImage("LIMITS.bmp");
}
PICOTEST_CASE(REVERSE_TESTS) {
    smGrid(W,H);
    PICOTEST_VERIFY( drawVLine(5,10,5), "Vertical Line reversed" )
    PICOTEST_VERIFY( drawHLine(5,10,5), "Horizontal Line reversed" )
    smGrid2NamedImage("REVERSE.bmp");
}